from typing import *
from google.cloud.storage import Client, Bucket
from google.oauth2 import service_account

import json

from typing import Optional

from .base import Base

class GS(Base):
    _creds: service_account.Credentials = None
    _project: str = None
    _bucket: Bucket = None

    def __init__(self, bucket: str, creds_path: Optional[str] = None):
        super().__init__()
        if creds_path is not None:
            self._creds = service_account.Credentials.from_service_account_file(creds_path)
            with open(creds_path, 'rt') as f:
                self._project = json.loads(f.read())['project_id']
        
            self._bucket = Client(self._project, self._creds).bucket(bucket)
        else:
            self._bucket = Client().bucket(bucket)
        
    
    def get(self, path: str) -> bytes:
        return self._bucket.get_blob(path).download_as_string()

    def put(self, path: str, content: bytes):
        self._bucket.blob(path).upload_from_string(content)

    def exists(self, path: str) -> bool:
        return self._bucket.get_blob(path) is not None
    
    def delete(self, path: str):
        blobs = self._bucket.list_blobs(prefix=path)
        for blob in blobs:
            blob.delete()


import hub


def test_gs():
    bucket = hub.gs('snark_waymo_open_dataset').connect()
    txt = bucket.blob_get('temporary_blob.txt').decode()
    print(txt)


if __name__ == '__main__':
    test_gs()


import hub

conn = hub.s3(
            bucket='waymo-dataset-upload', 
            aws_creds_filepath='.creds/aws.json'
            ).connect()

arr = conn.array_create('test/dynamic_array_1', shape=(10000, 11, 400, 7), chunk=(100, 11, 400, 7), dtype='uint8', dsplit=2)
print(arr.darray.shape)
print(arr[6, 2].shape)
arr.darray[0:100, 0:5] = (200, 6)
arr.darray[0:100, 5:11] = (300, 4)

print(arr[5, 3, :, :].shape)
print(arr[5, 6, :, :].shape)
print(arr[5, 4:6, :, :].shape)



import pytest
import hub
import numpy as np

arrname = 'test/array'

def main():
    test_array_create()
    test_array_open()
    test_array_delete()
    delete_item(arrname)

def _connect(bucket: str, creds: str):
    return hub.s3(bucket, aws_creds_filepath=creds).connect()

def connect():
    return _connect('snark-intelinair-export', '.creds/aws.json')

def delete_item(path: str, throw: bool = False):
    try:
        bucket = connect()
        bucket.delete(path)
    except Exception as ex:
        if throw:
            raise ex

def array_create(path: str):
    delete_item(path)
    bucket = connect()
    arr = bucket.array(path, shape=(100, 100, 2), chunk=(20, 20, 1), dtype=np.int32)
    return arr

def array_open(path: str):
    delete_item(path)
    array_create(path)
    bucket = connect()
    arr = bucket.open(path)
    arr[3,50] = np.array([1, 2])
    # print(arr[3, 50])
    return arr

def array_delete(path: str):
    delete_item(path)
    array_create(path)
    bucket = connect()
    bucket.delete(path)

def test_array_create():
    array_create(arrname)

def test_array_open():
    array_open(arrname)

def test_array_delete():
    array_delete(arrname)


if __name__ == "__main__":
    main()


from typing import *
import os, sys, time, random, json, itertools, uuid, traceback

from .bucket import Bucket
from . import storage
from .storage.retry_wrapper import RetryWrapper
from hub.utils.store_control import StoreControlClient

class Base():

    def __init__(self):
        pass

    def connect(self) -> Bucket:
        return Bucket(RetryWrapper(self._create_storage()))

    @staticmethod
    def _s3(
        bucket: Optional[str] = None, 
        aws_creds_filepath: Optional[str] = None, 
        aws_access_key_id: Optional[str] = None, 
        aws_secret_access_key: Optional[str] = None
        ) -> 'creds.Base':


        if aws_access_key_id is not None or aws_secret_access_key is not None:
            assert aws_access_key_id is not None
            assert aws_secret_access_key is not None
            assert bucket is not None
            assert aws_creds_filepath is None
            return S3(bucket, aws_access_key_id, aws_secret_access_key)
        elif aws_creds_filepath is not None:
            with open(aws_creds_filepath, 'r') as f:
                j = json.loads(f.read())
                if not bucket:
                    bucket = j['bucket']
                aws_access_key_id = j['aws_access_key_id']
                aws_secret_access_key = j['aws_secret_access_key']
                return S3(bucket, aws_access_key_id, aws_secret_access_key)
        else: 
            config = StoreControlClient.get_config()
            if bucket is None:
                bucket = config['BUCKET']
            
            aws_access_key_id = config['AWS_ACCESS_KEY_ID']
            aws_secret_access_key = config['AWS_SECRET_ACCESS_KEY']

            return S3(bucket, aws_access_key_id, aws_secret_access_key)

    def s3(
        self,
        bucket: Optional[str] = None, 
        aws_creds_filepath: Optional[str] = None, 
        aws_access_key_id: Optional[str] = None, 
        aws_secret_access_key: Optional[str] = None
        ) -> 'creds.Base':
            return Recursive(self, self.__class__._s3(bucket, aws_creds_filepath, aws_access_key_id, aws_secret_access_key))

    @staticmethod
    def _gs(bucket: str, creds_path: Optional[str] = None):
        return GS(bucket, creds_path)

    def gs(self, bucket: str, creds_path: Optional[str] = None) -> 'creds.Base':
        return Recursive(self, self.__class__._gs(bucket, creds_path))

    @staticmethod
    def _fs(dir: str) -> 'creds.Base':
        dir = os.path.expanduser(dir)
        
        if not os.path.exists(dir):
            os.makedirs(dir)
        
        return FS(dir)

    def fs(self, dir: str) -> 'creds.Base':
        return Recursive(self, self.__class__._fs(dir))

    def _create_storage(self) -> storage.Base:
        raise NotImplementedError()

class Recursive(Base):
    _core: 'creds.Base' = None
    _wrapper: 'creds.Base' = None

    def __init__(self, core: 'creds.Base', wrapper: 'creds.Base'):
        super().__init__()
        self._core = core
        self._wrapper = wrapper

    def _create_storage(self):
        return storage.Recursive(self._core._create_storage(), self._wrapper._create_storage())

class S3(Base):

    _bucket: str = None
    _aws_access_key_id: str = None
    _aws_secret_access_key: str = None

    def __init__(self, bucket: str, aws_access_key_id: str, aws_secret_access_key: str):
        super().__init__()
        self._bucket = bucket
        self._aws_access_key_id = aws_access_key_id
        self._aws_secret_access_key = aws_secret_access_key

    def _create_storage(self):
        return storage.S3(self._bucket, self._aws_access_key_id, self._aws_secret_access_key)
        

class GS(Base):
    _creds_path: str = None
    _bucket: str = None

    def __init__(self, bucket: str, creds_path: str):
        self._bucket = bucket
        if creds_path is not None:
            self._creds_path = os.path.expanduser(creds_path)
        else:
            self._creds_path = None

    def _create_storage(self):
        return storage.GS(self._bucket, self._creds_path)

class FS(Base):
    _dir: str = None

    def __init__(self, dir: str):
        super().__init__()
        self._dir = dir

    def _create_storage(self):
        return storage.FS(self._dir)



import hub
import torch
import torch.utils.data as data
import torchvision.transforms as transforms
    
class TorchDataset(data.Dataset):
    """ Dataset
    Args:
        dataset (hub.Dataset): hub dataset object 
    """

    def __init__(self, dataset, transforms=None):
        self.size = dataset.shape[0]
        self.key = dataset.key
        self.dataset = None
        self.batch = None
        self.indexbeta = 0
        self.batch_size = dataset.chunk_shape[0]

    def __getitem__(self, index):
        """
        Args:
            index (int): Index
        Returns:
            tuple: (dataset,...) 
        """
        # Load dataset for each worker
        if not self.dataset:
            self.dataset = hub.Dataset(key=self.key)

        # if self.batch is None or self.indexbeta == self.batch_size:
        #     self.indexbeta = 0
        #     self.batch = self.dataset[:self.batch_size]

        # output = list(map(lambda x:x[self.indexbeta], self.batch))
        # self.indexbeta += 1

        output = list(self.dataset[index])
        
        return (*output, )

    def __len__(self):
        return self.size

class TorchIterableDataset(data.IterableDataset):
    def __init__(self, dataset, transforms=None):
        self.size = dataset.shape[0]
        self.path = dataset._path
        self.storage = dataset._storage 
        self.dataset = None
        self.transform = transforms
        
    def _get_common_chunk(self, dataset):
        batch_sizes = map(lambda x: x[0], dataset.chunks.values())
        return min(batch_sizes)

    def _enumerate(self, dataset):
        """
        Seperate for each worker the id such that there is no conflict
        """
        worker_info = torch.utils.data.get_worker_info()
        if worker_info is None:
            for i, x in enumerate(dataset):
                yield x
                
        id = worker_info.id
        workers = worker_info.num_workers
        batch = self._get_common_chunk(dataset)

        # Be careful might contain bugs
        for i in range(dataset.shape[0]):
            if (i//batch)%workers == id:
                yield dataset[i]
        
    def __iter__(self):
        
        if self.dataset is None:
            self.dataset = hub.Dataset(self.path, self.storage)
            
        for x in self._enumerate(self.dataset):
            x = self.transform(x)
            yield (*list(x),)
            
    def __len__(self):
        return self.size


import hub
import torch
from torchvision import transforms
import torchvision.datasets as datasets


def test_pytorch():
    print('testing pytorch')

    # Create arrays
    images = hub.array((100000, 100, 100),
                       name='test/dataloaders:images', dtype='uint8')
    labels = hub.array(
        (100000, 1), name='test/dataloaders:labels', dtype='uint8')

    # Create dataset
    ds = hub.dataset({
        'images': images,
        'labels': labels
    }, name='test/loaders:dataset')

    # Transform to Pytorch
    train_dataset = ds.to_pytorch()

    # Create data loader
    train_loader = torch.utils.data.DataLoader(
        train_dataset, batch_size=32, num_workers=1,
        pin_memory=True
    )

    # Loop over attributes
    for i, (images, labels) in enumerate(train_loader):
        print(images.shape, labels.shape)
        assert (images.shape == (32, 100, 100))
        assert (labels.shape == (32, 1))
        break

    print('pass')


def test_keras():
    print('testing keras')
    ...
    print('not implemented')


def test_tensorflow():
    print('testing Tensorflow')
    ...
    print('Not implemented')


if __name__ == "__main__":
    test_pytorch()
    exit()
    # test_keras()
    # test_tensorflow()



import boto3
import botocore

from .base import Base

class S3(Base):
    def __init__(
        self, 
        bucket: str, 
        aws_access_key_id: str, 
        aws_secret_access_key: str):

        super().__init__()
        self._bucket_name = bucket
        self._client = boto3.client(
            's3', 
            aws_access_key_id = aws_access_key_id, 
            aws_secret_access_key = aws_secret_access_key, 
            config=botocore.config.Config(max_pool_connections=128))
        self._resource = boto3.resource(
            's3', 
            aws_access_key_id = aws_access_key_id, 
            aws_secret_access_key = aws_secret_access_key)
        self._bucket = self._resource.Bucket(bucket)

    def get(self, path: str) -> bytes:
        return self._bucket.Object(path).get()['Body'].read()

    def put(self, path: str, content: bytes) -> None:
        attrs = {
            'Bucket': self._bucket_name,
            'Body': content,
            'Key': path,
            'ContentType': ('application/octet-stream'),
        }
        self._client.put_object(**attrs)
    
    def exists(self, path: str) -> bool:
        return self.get_or_none(path) is not None
    
    def delete(self, path: str):
        self._bucket.objects.filter(Prefix=path).delete()

    def get_or_none(self, path):
        try:
            resp = self._client.get_object(
                Bucket=self._bucket_name,
                Key=path,
            )
            return resp['Body'].read()
        except botocore.exceptions.ClientError as err:
            if err.response['Error']['Code'] == 'NoSuchKey':
                return None
            else:
                raise err


import pytest
import hub
import numpy as np


def test_init():
    print('- Initialize array')
    shape = (10, 10, 10, 10)
    x = hub.array(shape, name="test/example:1", dtype='uint8')
    shape = np.array(shape)
    assert np.all(np.array(x.shape) == shape)
    print('passed')


def test_simple_upload_download():
    print('- Simple Chunk Upload and Download')
    x = hub.array((10, 10, 10, 10), name="test/example:1", dtype='uint8')
    x[0] = np.ones((1, 10, 10, 10), dtype='uint8')
    assert x[0].mean() == 1
    print('passed')


def test_multiple_upload_download():
    print('- Multiple Chunk Upload and Download')
    x = hub.array((10, 10, 10, 10), name="test/example:1", dtype='uint8')
    x[0:3] = np.ones((3, 10, 10, 10), dtype='uint8')
    assert x[0:3].mean() == 1
    print('passed')


def test_cross_chunk_upload_download():
    print('- Cross Chunk Upload and Download')
    x = hub.array((100, 100, 100), name="test/example:2", dtype='uint8')
    x[2:5, 0:10, 0:10] = np.ones((3, 10, 10), dtype='uint8')
    assert x[2:5, 0:10, 0:10].mean() == 1
    assert x[2:5, 10:, 10:].mean() == 0
    print('passed')


def test_broadcasting():
    print('- Broadcasting')
    x = hub.array((100, 100, 100), name="test/example:3", dtype='uint8')
    x[0, 0, 0] = 11
    assert x[0, 0, 0] == 11
    x[0] = 10
    assert x[0].mean() == 10
    x[1] = np.ones((100, 100), dtype='uint8')
    assert x[1].mean() == 1
    x[3, 90, :] = np.ones((1, 1, 100), dtype='uint8')
    assert x[3, 90].mean() == 1
    print('passed')


def test_chunk_shape():
    print('- Chunk shape')
    x = hub.array((100, 100, 100), name="test/example:3",
                  dtype='uint8', chunk_size=(10, 10, 10))
    x[0:10, 0:10, 0:10] = 0
    print('passed')


def test_load_array():
    print('- Loading arrays')
    x = hub.load(name="test/example:3")
    print(x.shape)
    print('passed')


def test_squeeze_array():
    print('- Squeezing arrays')
    x = hub.array((100, 100, 100), name="test/example:3", dtype='uint8')
    assert len(x[0].shape) == 2
    assert len(x[:1].shape) == 3
    assert len(x[:2, 0, :].sh  py2:
    image: python:2
    volumes:
       - ./:/workspace/
    command: bash -c "
          cd /workspace
          && pip install -e .
          && python -c 'import hub; hub.load(name=\"imagenet/image:train\")'"ape) == 2
    assert len(x[0, 0, :].shape) == 1
    assert x[0, 0, 0] == 0
    print('passed')


def test_dtypes():
    print('- Numpy dtypes arrays')
    x = hub.array((100,100,100), name="test/example:5", dtype=np.uint8)
    assert x.dtype == 'uint8'
    print('passed')

if __name__ == "__main__":
    print('Running Basic Tests')
    test_init()
    test_simple_upload_download()
    test_multiple_upload_download()
    test_cross_chunk_upload_download()
    test_broadcasting()
    test_chunk_shape()
    test_load_array()
    test_squeeze_array()
    test_dtypes()


from typing import *
import os, sys, io, traceback, time, random, json
from retrying import retry

from .base import Base


class RetryWrapper(Base):
    def __init__(self, internal: Base):
        self._internal = internal
    
    @retry(stop_max_attempt_number=3)
    def get(self, path): 
        return self._internal.get(path)
    
    @retry(stop_max_attempt_number=3)
    def put(self, path, content):
        self._internal.put(path, content)

    @retry(stop_max_attempt_number=3)
    def exists(self, path):
        return self._internal.exists(path) 
    
    @retry(stop_max_attempt_number=3)
    def delete(self, path):
        self._internal.delete(path)

    @retry(stop_max_attempt_number=3)
    def get_or_none(self, path):
        return self._internal.get_or_none(path)


from typing import *
import json, os, sys, time, random, uuid, io

from .array import Array, Props
from .storage import Base as Storage
from .dataset import Dataset, DatasetProps
import torch
import numpy as np


class Bucket():
    _storage: Storage = None

    def __init__(self, storage: Storage):
        self._storage = storage

    def array(
        self,
        name:str, 
        shape: Iterable[int],
        chunk: Iterable[int],
        dtype: Union[str, np.dtype],
        compress: str = 'default',
        compresslevel: float = 0.5,
        dsplit: int = None,
        ) -> Array:

        return self.array_create(name, shape, chunk, dtype, compress, compresslevel, dsplit)
    
    def dataset(self, name: str, components: Dict[str, str]):
        return self.dataset_create(name, components)
    
    def open(self, name: str):
        jsontext = self._storage.get_or_none(os.path.join(name, 'info.json'))
        if jsontext is None:
            return self.blob_get(name)
        else:
            d = json.loads(jsontext.decode())
            if 'compresslevel' in d:
                return self.array_open(name)
            else:
                return self.dataset_open(name)

    def delete(self, name: str):
        jsontext = self._storage.get_or_none(os.path.join(name, 'info.json'))
        if jsontext is None:
            if self._storage.get_or_none(name) is not None:
                self.blob_delete(name)
            else:
                raise Exception(f'Path f{name} is invalid for deletion')
        else:
            d = json.loads(jsontext.decode())
            if 'compresslevel' in d:
                self.array_delete(name)
            else:
                self.dataset_delete(name)
        
    def array_create(
        self, 
        name: str, 
        shape: Iterable[int], 
        chunk: Iterable[int], 
        dtype: Union[str, np.dtype], 
        compress: str = 'default', 
        compresslevel: float = 0.5, 
        dsplit: int = None,
        ) -> Array:
        
        shape = tuple(shape)
        chunk = tuple(chunk)

        overwrite = False
        props = Props()
        props.shape = shape
        props.chunk = chunk
        props.dtype = np.dtype(dtype).str
        props.compress = compress
        props.compresslevel = compresslevel
        if dsplit is not None:
            assert isinstance(dsplit, int)
            props.darray = 'darray'
            darray_path = os.path.join(name, props.darray)
            darray_shape = shape[:dsplit] + (len(shape) - dsplit,)
            arr = self.array_create(darray_path, darray_shape, darray_shape, 'int32')
            slices = tuple(map(lambda s: slice(0, s), shape[:dsplit]))
            arr[slices] = shape[dsplit:] 

        assert len(shape) == len(chunk)

        info_path = os.path.join(name, 'info.json')
        chunk_path = os.path.join(name, 'chunks')

        if overwrite or not self._storage.exists(info_path):
            self._storage.put(info_path, bytes(json.dumps(props.__dict__), 'utf-8'))

        if overwrite and self._storage.exists(chunk_path):
            self._storage.delete(chunk_path)

        return Array(name, self._storage)

    def array_open(self, name: str) -> Array:
        return Array(name, self._storage)

    def array_delete(self, name: str):
        self._storage.delete(name)

    def dataset_create(self, name: str, components: Dict[str, str]) -> Dataset:
        overwrite = False
        props = DatasetProps()
        props.paths = {}
        for key, comp in components.items():
            if isinstance(comp, Array):
                props.paths[key] = comp._path
            elif isinstance(comp, str):
                props.paths[key] = comp
            else:
                raise Exception(
                    'Input to the dataset is unknown: {}:{}'.format(k, value))

        dataset_path = os.path.join(name, 'info.json')

        if overwrite or not self._storage.exists(dataset_path):
            self._storage.put(dataset_path, bytes(json.dumps(props.__dict__), 'utf-8'))
        
        return Dataset(name, self._storage)

    def dataset_open(self, name: str) -> Dataset:
        return Dataset(name, self._storage)

    def dataset_delete(self, name: str):
        return self._storage.delete(name) 
    
    def blob_get(self, name: str, none_on_errors: bool = False) -> bytes:
        if none_on_errors:
            return self._storage.get_or_none(name)
        else:
            return self._storage.get(name)

    def blob_set(self, name: str, content: bytes):
        self._storage.put(name, content)

    def blob_delete(self, name: str) -> bytes: 
        self._storage.delete(name)
    
    def pytorch_model_get(self, name: str, none_on_errors: bool = False) -> dict:
        blob = self.blob_get(name, none_on_errors)
       
        if blob is None and not none_on_errors:
            raise Error('Error loading pytorch model')
        elif blob is None:
            return None
        else:
            f = io.BytesIO(blob)
            return torch.load(f)
    
    def pytorch_model_set(self, name: str, model: dict):
        f = io.BytesIO()
        torch.save(model, f)
        f.seek(0)
        self.blob_set(name, f.read())
    
    def pytorch_model_delete(self, name: str):
        self.blob_delete(name)


import hub
import pytest
import os

# Scenario 1. User has AWS credentials, no hub creds


def test_aws_wo_hub_creds():
    os.system('mv ~/.hub ~/.hub_arxiv')
    import hub
    x = hub.array((100, 100, 100), 'image/test:smth', dtype='uint8')
    print(x.shape)
    os.system('mv ~/.hub_arxiv ~/.hub')

# Scenario 2. User has Hub credentials, no aws creds

# Scenario 3. User has no credentials but wants to access public datasets


def test_public_access_no_creds():
    x = hub.load('imagenet')
    assert x[0].mean() == 1

# Scenario 4. User has no credentials and wants to create an array


def test_wo_aws_or_hub_creds():
    os.system('mv ~/.aws ~/.aws_arxiv')
    os.system('mv ~/.hub ~/.hub_arxiv')
    try:
        import hub
        x = hub.array((100, 100, 100), 'image/test:smth', dtype='uint8')
        print(x.shape)
    except Exception as err:
        print('pass', err)
        pass
    os.system('mv ~/.hub_arxiv ~/.hub')
    os.system('mv ~/.aws_arxiv ~/.aws')

# Scenario 5. Provided AWS credentials do not provide enough permission


if __name__ == "__main__":

    print('Running Basic Tests')
    test_aws_wo_hub_creds()
    test_wo_aws_or_hub_creds()
    test_public_access_no_creds()



import hub
import tensorflow

def test_tensorflow():
    print('testing tensorflow')

    # Create arrays
    images = hub.array((10, 100, 100), name='name1', dtype='uint8')
    labels = hub.array((10, 1), name='name2', dtype='uint8')

    # Create dataset
    ds = hub.dataset({
        'images': images,
        'labels': labels
    }, name='test/loaders:dataset')

    # Transform to Pytorch
    train_dataset = ds.to_tensorflow()

    assert len(train_dataset) == 10

    for image, label in train_dataset:
        assert len(image) == 100
        assert len(image[0]) == 100
        assert len(label) == 1        

    train_dataset = train_dataset.batch(32, drop_remainder=True);

    # Loop over attributes
    for _, (images, labels) in train_dataset.enumerate():
        # assert len(images) == 32
        # assert len(labels) == 32
        assert len(images[0]) == 100
        assert len(images[0][0]) == 100
        assert len(labels[0]) == 1
        break

    print('pass')



import hub


def test_mnist():
    print('test mnist')
    x = hub.load('mnist/mnist:train')
    assert x[59995].mean() == 32.910714285714285
    print('passed')


def test_imagenet():
    print('test imagenet')
    x = hub.load('imagenet')
    assert x[1000000].mean() == 163.95653688888888
    print('passed')


def test_dataset():
    x = hub.array((10000, 512, 512), name='test/example:input', dtype='uint8')
    y = hub.array((10000, 4), name='test/example:label', dtype='uint8')

    ds = hub.dataset({
        'input': x,
        'label': y
    }, name='test/dataset:train3')

    assert ds['input'].shape[0] == 10000   # return single array
    assert ds['label', 0].mean() == 0  # equivalent ds['train'][0]
    # return pair of arrays as long as dimensions requested are broadcastable
    assert ds[0][0].mean() == 0


def test_load_dataset():
    ds = hub.dataset(name='test/dataset:train3')  # return the dataset object
    assert ds.chunk_shape == [1]  # returns a dict of shapes
    assert ds.shape == [10000]


if __name__ == "__main__":
    print('Running Dataset Tests')
    test_mnist()
    test_imagenet()
    test_dataset()
    test_load_dataset()



from typing import *
import os, sys, time, random, uuid, traceback
from setuptools import find_packages, setup

project = "hub"
version = "0.4.1.2"

this_directory = os.path.abspath(os.path.dirname(__file__))
with open(os.path.join(this_directory, 'README.md')) as f:
    long_description = f.read()

with open(os.path.join(this_directory, 'requirements.txt'), 'r') as f:
    requirements = f.readlines()

setup(
    name=project,
    version=version,
    description="Snark Hub",
    long_description=long_description,
    long_description_content_type='text/markdown',
    author="Snark AI Inc.",
    author_email="support@snark.ai",
    url="https://github.com/snarkai/hub",
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,
    keywords="snark-hub",
    python_requires='>=3',
    install_requires=requirements,
    setup_requires=[],
    dependency_links=[],
    entry_points={
        "console_scripts": [
            "hub = hub:cli",
        ],
    },
    tests_require=[
        "pytest",
        "mock>=1.0.1",
    ],
)


import pytest
import hub
import numpy as np


def test_fs():
    print('- Test writing local to filesystem')
    shape = (10, 10, 10, 10)
    x = hub.array(shape, name="test/backend:1", dtype='uint8', backend='fs')
    shape = np.array(shape)
    assert np.all(np.array(x.shape) == shape)
    assert x[0, 0, 0, 0] == 0
    x[1] = 1
    assert x[1, 0, 0, 0] == 1
    print('passed')

def test_s3():
    print('- Test writing local to s3')
    shape = (10, 10, 10, 10)
    x = hub.array(shape, name="test/backend:2", dtype='uint8', backend='s3')
    shape = np.array(shape)
    assert np.all(np.array(x.shape) == shape)
    assert x[0, 0, 0, 0] == 0
    x[1] = 1
    assert x[1, 0, 0, 0] == 1
    print('passed')


def test_multiple():
    print('- Test writing local to filesystem and s3')
    shape = (10, 10, 10, 10)
    x = hub.array(shape, name="test/backend:3", dtype='uint8', backend=['fs', 's3'])
    shape = np.array(shape)
    assert np.all(np.array(x.shape) == shape)
    assert x[0, 0, 0, 0] == 0
    x[1] = 1
    assert x[1, 0, 0, 0] == 1
    print('passed')


def test_cache():
    print('- Test write to cache')
    shape = (10, 10, 10, 10)
    x = hub.array(shape, name="test/backend:4", dtype='uint8', backend=['fs', 's3'], caching=True)
    assert np.all(np.array(x.shape) == shape)
    assert x[0, 0, 0, 0] == 0
    x[1] = 1
    assert x[1, 0, 0, 0] == 1
    print('passed')
    
if __name__ == "__main__":
    test_fs()
    test_s3()
    test_multiple()



import hub
import tensorflow
from tensorflow.python.framework import dtypes
from tensorflow.python.framework import tensor_spec

class TensorflowDataset(tensorflow.data.Dataset):
    def __init__(self):
        pass

    def _inputs(self):
        raise NotImplementedError()

    @property
    def element_spec(self):
        raise NotImplementedError()
    
    def __iter__(self):
        raise NotImplementedError()

    def __len__(self):
        raise NotImplementedError()

    def apply(self, transformation_func):
        raise NotImplementedError()
        #return ApplyTensorflowDataset(self, transformation_func)

    def batch(self, batch_size, drop_remainder=False):
        return BatchTensorflowDataset(self, batch_size, drop_remainder=drop_remainder)

    def cache(self, filename=''):
        raise NotImplementedError()

    def concatenate(self, dataset):
        raise NotImplementedError()

    def enumerate(self, start = 0):
        i = 0
        for item in self:
            if i >= start:
                yield (i, item)
            i += 1

    def filter(self, predicate):
        raise NotImplementedError()
    
    def flat_map(self, map_func):
        raise NotImplementedError()

    # def interleave(self, map_func, cycle_length=tensorflow.data.AUTOTUNE, block_length=1, num_parallel_calls=None):
    #     pass

    def map(self, map_func, num_parallel_calls=None):
        raise NotImplementedError()
    
    def options(self):
        raise NotImplementedError()

    def padded_batch(self, batch_size, padded_shapes, padding_values=None, drop_remainder=False):
        raise NotImplementedError()

    def prefetch(self, buffer_size):
        raise NotImplementedError()

    def reduce(self, initial_state, reduce_func):
        raise NotImplementedError()

    def repeat(self, count=None):
        raise NotImplementedError()

    def shard(self, num_shards, index):
        raise NotImplementedError()

    def shuffle(self, buffer_size, seed=None, reshuffle_each_iteration=None):
        raise NotImplementedError()

    def skip(self, count):
        raise NotImplementedError()

    def take(self, count):
        raise NotImplementedError()

    def window(self, size, shift=None, stride=1,drop_remainder=False):
        raise NotImplementedError()

    def with_options(options):
        raise NotImplementedError()

class HubTensorflowDataset(TensorflowDataset):
    def __init__(self, hub_dataset):
        self.__hub_dataset = hub_dataset

    def __len__(self):
        return self.__hub_dataset.shape[0]

    def __iter__(self):
        # if self.__dataset is None:
        #     self.__dataset = hub.Dataset(key = self._hub_dataset.key)
        
        for i in self.__hub_dataset:
            print(i)
            yield (*list(i),)

# class ApplyTensorflowDataset(TensorflowDataset):
#     def __init__(self, wrappee, transformation_func):
#         self.__wrappee = wrappee
#         self.__transformation_func = transformation_func
    
#     def __iter__(self):
#         pass
    

class BatchTensorflowDataset(TensorflowDataset):
    def __init__(self, wrappee, batch_size, drop_remainder=False):
        self.__wrappee = wrappee
        self.__batch_size = batch_size
        self.__drop_remainder = drop_remainder
    
    def __iter__(self):
        res = None
        cnt = 0
        for item in self.__wrappee:
            if res is None:
                res = [[]] * len(item)
            
            if cnt > 0 and cnt % self.__batch_size == 0:
                yield (*res,)
                res = [[]] * len(res)
            
            cnt += 1
            for j in range(0, len(item)):
                res[j].append(item[j])
        
        if not self.__drop_remainder and res != None and len(res[0]) > 0:
            yield (*res,)

    def __len__(self):
        ans = len(self.__wrappee) / self.__batch_size
        if not self.__drop_remainder and len(self.__wrappee) % self.__batch_size > 0:
            return ans + 1
        else:
            return ans
    


from typing import Optional

class Base():
    def get(self, path: str) -> bytes:
        raise NotImplementedError

    def put(self, path: str, content: bytes):
        raise NotImplementedError

    def exists(self, path: str) -> bool:
        raise NotImplementedError
    
    def delete(self, path: str):
        raise NotImplementedError()

    def get_or_none(self, path: str) -> Optional[bytes]:
        if self.exists(path):
           return self.get(path)
        else:
            return None